#include "kernel.h"
/* #include "t_syslog.h" */
#include "tTDR_tecsgen.h"

#include "syslog.c"

/* 呼び口関数 */
/*
 * call port : cChannel  signature: sChannel
 *   ER             cChannel_open( const int8_t* arg, int16_t size );
 *   ER             cChannel_close();
 *   ER             cChannel_send( const int8_t* buf, int16_t size );
 *   ER             cChannel_receive( const int8_t* buf, int16_t size );
 */

/* 受け口関数 */
/*
 * entry port: eTDR
 * signature:  sTDR
 */
/*
 * name:         eTDR_open
 * global_name:  tTDR_eTDR_open
 */
ER             eTDR_open( tTDR_IDX idx, const int8_t* arg, int16_t size, TMO tmo )
{
  CELLCB *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }
  return cChannel_open( arg, size, tmo );
}

/*
 * name:         eTDR_close
 * global_name:  tTDR_eTDR_close
 */
ER             eTDR_close( tTDR_IDX idx)
{
  CELLCB *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }
  return cChannel_close();
}

/*
 * name:         eTDR_reset
 * global_name:  tTDR_eTDR_reset
 */
ER             eTDR_reset( tTDR_IDX idx)
{
  CELLCB *p_cellcb;
  int32_t    sz, times;
  ER         er;

  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_reset();
}


/*
 * name:         eTDR_sop
 * global_name:  tTDR_eTDR_sop
 */
ER             eTDR_sop( tTDR_IDX idx, int no )
{
	ER      er;
    int32_t magic;
	CELLCB *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

#ifdef DEBUG
	syslog(LOG_INFO, "eTDR_sop(no=%d)", no);
#endif

    if( no == 1 )
		magic = /*TDR_SOP_MAGIC1*/ 0x9951672A;
	else
		magic = TDR_SOP_MAGIC2;

	/* SOP パケットの送信 */
	if( (er = eTDR_put_int32( idx, magic )) != E_OK )
		return er;

	return E_OK;
}

/*
 * name:         eTDR_is_sop
 * global_name:  tTDR_eTDR_is_sop
 */
ER             eTDR_is_sop( tTDR_IDX idx, int no )
{
	CELLCB *p_cellcb;
	ER     er;
	int32_t             magic;

	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return E_ID;
	}

#ifdef DEBUG
	syslog(LOG_INFO, "eTDR_is_sop()");
#endif

	if( (er=eTDR_get_int32( idx, &magic )) != E_OK ){
		return ERCD( E_RPC, er );
	} else if( (no == 1 && magic == TDR_SOP_MAGIC1) || (no == 2 && magic == TDR_SOP_MAGIC2) ){
		return E_OK;
	} else {
		return ERCD( E_RPC, E_MAGIC );
	}

	return E_OK;
}

/*
 * name:         eTDR_eop
 * global_name:  tTDR_eTDR_eop
 */
ER             eTDR_eop( tTDR_IDX idx)
{
  CELLCB *p_cellcb;
  ER     er;

  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

#ifdef DEBUG
  syslog(LOG_INFO, "eTDR_eop()");
#endif

  /* EOP パケットの受信 */
  if( (er = eTDR_put_int32( idx, (int32_t)TDR_EOP_MAGIC )) != E_OK )
	  return er;

  return cChannel_flush();
}

/*
 * name:         eTDR_is_eop
 * global_name:  tTDR_eTDR_is_eop
 */
ER             eTDR_is_eop( tTDR_IDX idx)
{
  CELLCB     *p_cellcb;
  int32_t    magic;
  ER         er, er2;

  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

#ifdef DEBUG
	syslog(LOG_INFO, "eTDR_is_eop()");
#endif
/* mikan cChannel EOF get_* */
/* mikan magic EOF */

  if( (er=eTDR_get_int32( idx, &magic )) != E_OK ){
	  er = ERCD( E_RPC, er );
  } else if( magic == TDR_EOP_MAGIC ){
	  er = E_OK;
  } else {
	  er = ERCD( E_RPC, E_MAGIC );
  }
#ifdef DEBUG
	syslog(LOG_INFO, "eTDR_is_eop(): %x", er);
#endif
  er2 = cChannel_end_receive();
  return er != E_OK ? er: er2;
}

/*
 * name:         eTDR_set_tmo
 * global_name:  tTDR_eTDR_set_tmo
 */
ER             eTDR_set_tmo( tTDR_IDX idx, TMO tmo)
{
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  VAR_rtmo = tmo;
  return E_OK;
}

/*
 * name:         eTDR_put_int8_t
 * global_name:  tTDR_eTDR_put_int8_t
 */
ER             eTDR_put_int8_t( tTDR_IDX idx, int8_t in)
{
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_int16
 * global_name:  tTDR_eTDR_put_int16
 */
ER             eTDR_put_int16( tTDR_IDX idx, int16_t in)
{
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_int32
 * global_name:  tTDR_eTDR_put_int32
 */
ER             eTDR_put_int32( tTDR_IDX idx, int32_t in)
{
  CELLCB     *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_int64
 * global_name:  tTDR_eTDR_put_int64
 */
ER             eTDR_put_int64( tTDR_IDX idx, int64_t in)
{
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_int128
 * global_name:  tTDR_eTDR_put_int128
 */
ER             eTDR_put_int128( tTDR_IDX idx, int128_t in)
{
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_get_int8_t
 * global_name:  tTDR_eTDR_get_int8_t
 */
ER             eTDR_get_int8_t( tTDR_IDX idx, int8_t* out)
{
  CELLCB      *p_cellcb;
  ER_UINT     er_sz;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
  return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_int16
 * global_name:  tTDR_eTDR_get_int16
 */
ER             eTDR_get_int16( tTDR_IDX idx, int16_t* out)
{
  ER_UINT     er_sz;
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
  return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_int32
 * global_name:  tTDR_eTDR_get_int32
 */
ER             eTDR_get_int32( tTDR_IDX idx, int32_t* out)
{
  ER_UINT     er_sz;
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
  return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_int64
 * global_name:  tTDR_eTDR_get_int64
 */
ER             eTDR_get_int64( tTDR_IDX idx, int64_t* out)
{
  ER_UINT     er_sz;
  CELLCB *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
  return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_int128
 * global_name:  tTDR_eTDR_get_int128
 */
ER             eTDR_get_int128( tTDR_IDX idx, int128_t* out)
{
  ER_UINT     er_sz;
  CELLCB      *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
  return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_put_char
 * global_name:  tTDR_eTDR_put_char
 */
ER             eTDR_put_char( tTDR_IDX idx, char_t in)
{
  CELLCB     *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_short
 * global_name:  tTDR_eTDR_put_short
 */
ER             eTDR_put_short( tTDR_IDX idx, short in)
{
  CELLCB     *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_int
 * global_name:  tTDR_eTDR_put_int
 */
ER             eTDR_put_int( tTDR_IDX idx, int_t in)
{
  CELLCB     *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_put_long
 * global_name:  tTDR_eTDR_put_long
 */
ER             eTDR_put_long( tTDR_IDX idx, long_t in)
{
  CELLCB *p_cellcb;
  if( tTDR_VALID_IDX( idx ) ){
    p_cellcb = GET_CELLCB(idx);
  }else{
    return ERCD( E_RPC, E_ID );
  }

  return cChannel_send( (int8_t *)&in, (int16_t)sizeof( in ), VAR_rtmo );
}

/*
 * name:         eTDR_get_char
 * global_name:  tTDR_eTDR_get_char
 */
ER             eTDR_get_char( tTDR_IDX idx, char_t* out)
{
	ER_UINT    er_sz;
	CELLCB     *p_cellcb;
    if( tTDR_VALID_IDX( idx ) ){
         p_cellcb = GET_CELLCB(idx);
    }else{
         return ERCD( E_RPC, E_ID );
    }

    er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
    return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_short
 * global_name:  tTDR_eTDR_get_short
 */
ER             eTDR_get_short( tTDR_IDX idx, short* out)
{
	ER_UINT    er_sz;
	CELLCB     *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return ERCD( E_RPC, E_ID );
	}

	er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
	return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_int
 * global_name:  tTDR_eTDR_get_int
 */
ER             eTDR_get_int( tTDR_IDX idx, int_t* out)
{
	ER_UINT    er_sz;
	CELLCB     *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return ERCD( E_RPC, E_ID );
	}

	er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
	return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_get_long
 * global_name:  tTDR_eTDR_get_long
 */
ER             eTDR_get_long( tTDR_IDX idx, long_t* out)
{
	ER_UINT    er_sz;
	CELLCB     *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return ERCD( E_RPC, E_ID );
	}

	er_sz = cChannel_receive( (int8_t *)out, (int16_t)sizeof( *out ), VAR_rtmo );
	return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *out ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

/*
 * name:         eTDR_put_intptr
 * global_name:  tTDR_eTDR_put_intptr
 */
ER             eTDR_put_intptr( tTDR_IDX idx, const void* ptr)
{
	CELLCB     *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return ERCD( E_RPC, E_ID );
	}

	return cChannel_send( (int8_t *)&ptr, (int16_t)sizeof( ptr ), VAR_rtmo );
}

/*
 * name:         eTDR_get_intptr
 * global_name:  tTDR_eTDR_get_intptr
 */
ER             eTDR_get_intptr( tTDR_IDX idx, void** ptr)
{
	ER_UINT    er_sz;
	CELLCB     *p_cellcb;
	if( tTDR_VALID_IDX( idx ) ){
		p_cellcb = GET_CELLCB(idx);
	}else{
		return ERCD( E_RPC, E_ID );
	}

	er_sz = cChannel_receive( (int8_t *)ptr, (int16_t)sizeof( *ptr ), VAR_rtmo );
	return  er_sz < 0 ? er_sz: ( er_sz == sizeof( *ptr ) ? E_OK : ERCD( E_RPC, E_BOVR ) );
}

